@@ -14,7 +14,18 @@ class Api(object):
     should likely be moved to PublicApi"""
 
     def __init__(self, *args, **kwargs):
-        self.api = InternalApi(*args, **kwargs)
+        self._api_args = args
+        self._api_kwargs = kwargs
+        self._api = None
+
+    @property
+    def api(self):
+        # This is a property in order ot delay construction of Internal API
+        # for as long as possible. If constructed in constructor, then the
+        # whole InternalAPI is started when simply importing wandb.
+        if self._api is None:
+            self._api = InternalApi(*self._api_args, **self._api_kwargs)
+        return self._api
 
     @property
     def api_key(self):
