@@ -113,25 +113,28 @@ class PolicyLive(FileEventHandler):
         else:
             return 20 * 60
 
-    def last_uploaded(self):
+    def should_update(self):
         if self._last_uploaded_time:
             # Check rate limit by time elapsed
             time_elapsed = time.time() - self._last_uploaded_time
+            # if more than 15 seconds has passed potentially upload it
             if time_elapsed < self.RATE_LIMIT_SECONDS:
-                return time_elapsed
+                return False
+
             # Check rate limit by size increase
             if float(self._last_uploaded_size) > 0:
                 size_increase = self.current_size / float(self._last_uploaded_size)
                 if size_increase < self.RATE_LIMIT_SIZE_INCREASE:
-                    return time_elapsed
-        return 0
+                    return False
+            return time_elapsed > self.min_wait_for_size(self.current_size)
+
+        # if the file has never been uploaded, we'll upload it
+        return True
 
     def on_modified(self, force=False):
         if self.current_size == 0:
             return 0
-
-        time_elapsed = self.last_uploaded()
-        if not self.synced or time_elapsed > self.min_wait_for_size(self.current_size):
+        if not self.synced and self.should_update():
             self.save_file()
         # if the run is finished, or wandb.save is called explicitly save me
         elif force and not self.synced:
