@@ -11,6 +11,9 @@ import sys
 import wandb
 from wandb import env
 from wandb import util
+from wandb.errors import Error
+
+from requests import HTTPError
 
 logger = logging.getLogger(__name__)
 
@@ -43,7 +46,7 @@ class Retry(object):
     MAX_SLEEP_SECONDS = 5 * 60
 
     def __init__(self, call_fn, retry_timedelta=None, num_retries=None, check_retry_fn=lambda e: True,
-                 retryable_exceptions=None, error_prefix="Network error"):
+                 retryable_exceptions=None, error_prefix="Network error", retry_callback=None):
         self._call_fn = call_fn
         self._check_retry_fn = check_retry_fn
         self._error_prefix = error_prefix
@@ -54,6 +57,7 @@ class Retry(object):
         if self._retryable_exceptions is None:
             self._retryable_exceptions = (TransientException,)
         self._index = 0
+        self.retry_callback = retry_callback
 
     @property
     def num_iters(self):
@@ -97,8 +101,7 @@ class Retry(object):
                 # Only print resolved attempts once every minute
                 if self._num_iter > 2 and now - self._last_print > datetime.timedelta(minutes=1):
                     self._last_print = datetime.datetime.now()
-                    wandb.termlog('{} resolved after {}, resuming normal operation.'.format(
-                        self._error_prefix, datetime.datetime.now() - start_time))
+                    self.retry_callback(200, "{} resolved after {}, resuming normal operation.".format(self._error_prefix, datetime.datetime.now() - start_time))
                 return result
             except self._retryable_exceptions as e:
                 # if the secondary check fails, re-raise
@@ -109,9 +112,13 @@ class Retry(object):
                     raise
                 if self._num_iter == 2:
                     logger.exception('Retry attempt failed:')
-                    wandb.termlog(
-                        '{} ({}), entering retry loop. See {} for full traceback.'.format(
-                            self._error_prefix, e.__class__.__name__, util.get_log_file_path()))
+                    if isinstance(e, HTTPError) and e.response is not None and self.retry_callback is not None:
+                        self.retry_callback(e.response.status_code, e.response.text)
+                    else:
+                        # todo: would like to catch other errors, eg wandb.errors.Error, ConnectionError etc
+                        # but some of these can be raised before the retry handler thread (RunStatusChecker) is
+                        # spawned in wandb_init
+                        wandb.termlog("{} ({}), entering retry loop.".format(self._error_prefix, e.__class__.__name__))
                 # if wandb.env.is_debug():
                 #     traceback.print_exc()
             time.sleep(sleep + random.random() * 0.25 * sleep)
