@@ -4,6 +4,7 @@ import json
 import logging
 import numbers
 import os
+import re
 import shutil
 import sys
 
@@ -34,6 +35,7 @@ if wandb.TYPE_CHECKING:
     )
 
     if TYPE_CHECKING:  # pragma: no cover
+        from .interface.artifacts import ArtifactEntry
         from .wandb_artifacts import Artifact as LocalArtifact
         from .wandb_run import Run as LocalRun
         from wandb.apis.public import Artifact as PublicArtifact
@@ -79,7 +81,16 @@ class _WBValueArtifactSource(object):
     artifact: "PublicArtifact"
     name: Optional[str]
 
-    def __init__(self, artifact: "PublicArtifact", name: str = None) -> None:
+    def __init__(self, artifact: "PublicArtifact", name: Optional[str] = None) -> None:
+        self.artifact = artifact
+        self.name = name
+
+
+class _WBValueArtifactTarget(object):
+    artifact: "LocalArtifact"
+    name: Optional[str]
+
+    def __init__(self, artifact: "LocalArtifact", name: Optional[str] = None) -> None:
         self.artifact = artifact
         self.name = name
 
@@ -95,14 +106,16 @@ class WBValue(object):
 
     # Class Attributes
     _type_mapping: ClassVar[Optional["TypeMappingType"]] = None
-    # override artifact_type to indicate the type which the subclass deserializes
-    artifact_type: ClassVar[Optional[str]] = None
+    # override _log_type to indicate the type which the subclass deserializes
+    _log_type: ClassVar[Optional[str]] = None
 
     # Instance Attributes
-    artifact_source: Optional[_WBValueArtifactSource]
+    _artifact_source: Optional[_WBValueArtifactSource]
+    _artifact_target: Optional[_WBValueArtifactTarget]
 
     def __init__(self) -> None:
-        self.artifact_source = None
+        self._artifact_source = None
+        self._artifact_target = None
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         """Serializes the object into a JSON blob, using a run or artifact to store additional data.
@@ -140,10 +153,10 @@ class WBValue(object):
             filetype (str, optional): the filetype to use. Defaults to "json".
 
         Returns:
-            str: a filename which is suffixed with it's `artifact_type` followed by the filetype
+            str: a filename which is suffixed with it's `_log_type` followed by the filetype
         """
-        if cls.artifact_type is not None:
-            suffix = cls.artifact_type + "." + filetype
+        if cls._log_type is not None:
+            suffix = cls._log_type + "." + filetype
         else:
             suffix = filetype
         if not name.endswith(suffix):
@@ -171,14 +184,14 @@ class WBValue(object):
         class_option = WBValue.type_mapping().get(json_obj["_type"])
         if class_option is not None:
             obj = class_option.from_json(json_obj, source_artifact)
-            obj.set_artifact_source(source_artifact)
+            obj._set_artifact_source(source_artifact)
             return obj
 
         return None
 
     @staticmethod
     def type_mapping() -> "TypeMappingType":
-        """Returns a map from `artifact_type` to subclass. Used to lookup correct types for deserialization.
+        """Returns a map from `_log_type` to subclass. Used to lookup correct types for deserialization.
 
         Returns:
             dict: dictionary of str:class
@@ -190,8 +203,8 @@ class WBValue(object):
             while len(frontier) > 0:
                 class_option = frontier.pop()
                 explored.add(class_option)
-                if class_option.artifact_type is not None:
-                    WBValue._type_mapping[class_option.artifact_type] = class_option
+                if class_option._log_type is not None:
+                    WBValue._type_mapping[class_option._log_type] = class_option
                 for subclass in class_option.__subclasses__():
                     if subclass not in explored:
                         frontier.append(subclass)
@@ -203,8 +216,50 @@ class WBValue(object):
     def __ne__(self, other: object) -> bool:
         return not self.__eq__(other)
 
-    def set_artifact_source(self, artifact: "PublicArtifact", name: str = None) -> None:
-        self.artifact_source = _WBValueArtifactSource(artifact, name)
+    def to_data_array(self) -> List[Any]:
+        """Converts the object to a list of primitives representing the underlying data"""
+        raise NotImplementedError
+
+    def _set_artifact_source(
+        self, artifact: "PublicArtifact", name: Optional[str] = None
+    ) -> None:
+        assert (
+            self._artifact_source is None
+        ), "Cannot update artifact_source. Existing source: {}/{}".format(
+            self._artifact_source.artifact, self._artifact_source.name
+        )
+        self._artifact_source = _WBValueArtifactSource(artifact, name)
+
+    def _set_artifact_target(
+        self, artifact: "LocalArtifact", name: Optional[str] = None
+    ) -> None:
+        assert (
+            self._artifact_target is None
+        ), "Cannot update artifact_target. Existing target: {}/{}".format(
+            self._artifact_target.artifact, self._artifact_target.name
+        )
+        self._artifact_target = _WBValueArtifactTarget(artifact, name)
+
+    def _get_artifact_reference_entry(self) -> Optional["ArtifactEntry"]:
+        ref_entry = None
+        # If the object is coming from another artifact
+        if self._artifact_source and self._artifact_source.name:
+            ref_entry = self._artifact_source.artifact.get_path(
+                type(self).with_suffix(self._artifact_source.name)
+            )
+        # Else, if the object is destined for another artifact
+        elif (
+            self._artifact_target
+            and self._artifact_target.name
+            and self._artifact_target.artifact._logged_artifact is not None
+        ):
+            # Currently, we do not have a way to obtain a reference URL without waiting for the
+            # upstream artifact to be logged. This implies that this only works online as well.
+            self._artifact_target.artifact.wait()
+            ref_entry = self._artifact_target.artifact.get_path(
+                type(self).with_suffix(self._artifact_target.name)
+            )
+        return ref_entry
 
 
 class Histogram(WBValue):
@@ -237,6 +292,7 @@ class Histogram(WBValue):
     """
 
     MAX_LENGTH: int = 512
+    _log_type = "histogram"
 
     def __init__(
         self,
@@ -277,7 +333,7 @@ class Histogram(WBValue):
             raise ValueError("len(bins) must be len(histogram) + 1")
 
     def to_json(self, run: Union["LocalRun", "LocalArtifact"] = None) -> dict:
-        return {"_type": "histogram", "values": self.histogram, "bins": self.bins}
+        return {"_type": self._log_type, "values": self.histogram, "bins": self.bins}
 
     def __sizeof__(self) -> int:
         """This returns an estimated size in bytes, currently the factor of 1.7
@@ -374,8 +430,6 @@ class Media(WBValue):
         # Following assertion required for mypy
         assert self._run is not None
 
-        base_path = os.path.join(self._run.dir, self.get_media_subdir())
-
         if self._extension is None:
             _, extension = os.path.splitext(os.path.basename(self._path))
         else:
@@ -386,7 +440,7 @@ class Media(WBValue):
 
         file_path = _wb_filename(key, step, id_, extension)
         media_path = os.path.join(self.get_media_subdir(), file_path)
-        new_path = os.path.join(base_path, file_path)
+        new_path = os.path.join(self._run.dir, media_path)
         util.mkdir_exists_ok(os.path.dirname(new_path))
 
         if self._is_tmp:
@@ -443,6 +497,9 @@ class Media(WBValue):
                     "size": self._size,
                 }
             )
+            artifact_entry = self._get_artifact_reference_entry()
+            if artifact_entry is not None:
+                json_obj["artifact_path"] = artifact_entry.ref_url()
         elif isinstance(run, artifact_class):
             if self.file_is_set():
                 # The following two assertions are guaranteed to pass
@@ -469,18 +526,18 @@ class Media(WBValue):
 
                     # if not, check to see if there is a source artifact for this object
                     if (
-                        self.artifact_source
+                        self._artifact_source
                         is not None
-                        # and self.artifact_source.artifact != artifact
+                        # and self._artifact_source.artifact != artifact
                     ):
-                        default_root = self.artifact_source.artifact._default_root()
+                        default_root = self._artifact_source.artifact._default_root()
                         # if there is, get the name of the entry (this might make sense to move to a helper off artifact)
                         if self._path.startswith(default_root):
                             name = self._path[len(default_root) :]
                             name = name.lstrip(os.sep)
 
                         # Add this image as a reference
-                        path = self.artifact_source.artifact.get_path(name)
+                        path = self._artifact_source.artifact.get_path(name)
                         artifact.add_reference(path.ref_url(), name=name)
                     elif isinstance(self, Audio) and Audio.path_is_reference(
                         self._path
@@ -493,7 +550,7 @@ class Media(WBValue):
                         name = entry.path
 
                 json_obj["path"] = name
-            json_obj["_type"] = self.artifact_type
+            json_obj["_type"] = self._log_type
         return json_obj
 
     @classmethod
@@ -557,7 +614,7 @@ class Object3D(BatchableMedia):
     SUPPORTED_TYPES: ClassVar[Set[str]] = set(
         ["obj", "gltf", "glb", "babylon", "stl", "pts.json"]
     )
-    artifact_type: ClassVar[str] = "object3D-file"
+    _log_type: ClassVar[str] = "object3D-file"
 
     def __init__(
         self, data_or_path: Union["np.ndarray", str, "TextIO"], **kwargs: str
@@ -676,7 +733,7 @@ class Object3D(BatchableMedia):
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         json_dict = super(Object3D, self).to_json(run_or_artifact)
-        json_dict["_type"] = Object3D.artifact_type
+        json_dict["_type"] = Object3D._log_type
 
         _, artifact_class = _safe_sdk_import()
 
@@ -731,6 +788,7 @@ class Molecule(BatchableMedia):
     SUPPORTED_TYPES = set(
         ["pdb", "pqr", "mmcif", "mcif", "cif", "sdf", "sd", "gro", "mol2", "mmtf"]
     )
+    _log_type = "molecule-file"
 
     def __init__(self, data_or_path: Union[str, "TextIO"], **kwargs: str) -> None:
         super(Molecule, self).__init__()
@@ -780,7 +838,7 @@ class Molecule(BatchableMedia):
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         json_dict = super(Molecule, self).to_json(run_or_artifact)
-        json_dict["_type"] = "molecule-file"
+        json_dict["_type"] = self._log_type
         if self._caption:
             json_dict["caption"] = self._caption
         return json_dict
@@ -824,7 +882,7 @@ class Html(BatchableMedia):
             to False the HTML will pass through unchanged.
     """
 
-    artifact_type = "html-file"
+    _log_type = "html-file"
 
     def __init__(self, data: Union[str, "TextIO"], inject: bool = True) -> None:
         super(Html, self).__init__()
@@ -879,7 +937,7 @@ class Html(BatchableMedia):
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         json_dict = super(Html, self).to_json(run_or_artifact)
-        json_dict["_type"] = "html-file"
+        json_dict["_type"] = self._log_type
         return json_dict
 
     @classmethod
@@ -926,7 +984,7 @@ class Video(BatchableMedia):
         format: (string) format of video, necessary if initializing with path or io object.
     """
 
-    artifact_type = "video-file"
+    _log_type = "video-file"
     EXTS = ("gif", "mp4", "webm", "ogg")
     _width: Optional[int]
     _height: Optional[int]
@@ -1021,7 +1079,7 @@ class Video(BatchableMedia):
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         json_dict = super(Video, self).to_json(run_or_artifact)
-        json_dict["_type"] = "video-file"
+        json_dict["_type"] = self._log_type
 
         if self._width is not None:
             json_dict["width"] = self._width
@@ -1137,7 +1195,7 @@ class ImageMask(Media):
     Wandb class for image masks, useful for segmentation tasks
     """
 
-    artifact_type = "mask"
+    _log_type = "mask"
 
     def __init__(self, val: dict, key: str) -> None:
         """
@@ -1229,7 +1287,7 @@ class ImageMask(Media):
 
     @classmethod
     def type_name(cls: Type["ImageMask"]) -> str:
-        return "mask"
+        return cls._log_type
 
     def validate(self, val: dict) -> bool:
         np = util.get_module(
@@ -1267,7 +1325,9 @@ class BoundingBoxes2D(JSONMetadata):
     Wandb class for 2D bounding boxes
     """
 
-    artifact_type = "bounding-boxes"
+    _log_type = "bounding-boxes"
+    # TODO: when the change is made to have this produce a dict with a _type, define
+    # it here as _log_type, associate it in to_json
 
     def __init__(self, val: dict, key: str) -> None:
         """
@@ -1414,7 +1474,7 @@ class BoundingBoxes2D(JSONMetadata):
 
 
 class Classes(Media):
-    artifact_type = "classes"
+    _log_type = "classes"
 
     _class_set: Sequence[dict]
 
@@ -1446,7 +1506,7 @@ class Classes(Media):
         # In all other cases, artifact should be a true artifact.
         if run_or_artifact is not None:
             json_obj = super(Classes, self).to_json(run_or_artifact)
-        json_obj["_type"] = Classes.artifact_type
+        json_obj["_type"] = Classes._log_type
         json_obj["class_set"] = self._class_set
         return json_obj
 
@@ -1481,7 +1541,7 @@ class Image(BatchableMedia):
     # PIL limit
     MAX_DIMENSION = 65500
 
-    artifact_type = "image-file"
+    _log_type = "image-file"
 
     format: Optional[str]
     _grouping: Optional[str]
@@ -1586,7 +1646,8 @@ class Image(BatchableMedia):
         self._sha256 = wbimage._sha256
         self._size = wbimage._size
         self.format = wbimage.format
-        self.artifact_source = wbimage.artifact_source
+        self._artifact_source = wbimage._artifact_source
+        self._artifact_target = wbimage._artifact_target
 
         # We do not want to implicitly copy boxes or masks, just the image-related data.
         # self._boxes = wbimage._boxes
@@ -1652,7 +1713,7 @@ class Image(BatchableMedia):
             _masks = {}
             for key in masks:
                 _masks[key] = ImageMask.from_json(masks[key], source_artifact)
-                _masks[key].set_artifact_source(source_artifact)
+                _masks[key]._set_artifact_source(source_artifact)
                 _masks[key]._key = key
 
         boxes = json_obj.get("boxes")
@@ -1696,7 +1757,7 @@ class Image(BatchableMedia):
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         json_dict = super(Image, self).to_json(run_or_artifact)
-        json_dict["_type"] = Image.artifact_type
+        json_dict["_type"] = Image._log_type
         json_dict["format"] = self.format
 
         if self._width is not None:
@@ -1926,6 +1987,14 @@ class Image(BatchableMedia):
                 and self._classes == other._classes
             )
 
+    def to_data_array(self) -> List[Any]:
+        res = []
+        if self._image is not None:
+            data = list(self._image.getdata())
+            for i in range(self._image.height):
+                res.append(data[i * self._image.width : (i + 1) * self._image.width])
+        return res
+
 
 class Plotly(Media):
     """
@@ -1935,6 +2004,8 @@ class Plotly(Media):
         val: matplotlib or plotly figure
     """
 
+    _log_type = "plotly-file"
+
     @classmethod
     def make_plot_media(
         cls: Type["Plotly"], val: Union["plotly.Figure", "matplotlib.artist.Artist"]
@@ -1973,7 +2044,7 @@ class Plotly(Media):
 
     def to_json(self, run_or_artifact: Union["LocalRun", "LocalArtifact"]) -> dict:
         json_dict = super(Plotly, self).to_json(run_or_artifact)
-        json_dict["_type"] = "plotly-file"
+        json_dict["_type"] = self._log_type
         return json_dict
 
 
@@ -2052,6 +2123,19 @@ def val_to_json(
     if isinstance(val, WBValue):
         assert run
         if isinstance(val, Media) and not val.is_bound():
+            if hasattr(val, "_log_type") and val._log_type == "table":
+                # Special conditional to log tables as artifact entries as well.
+                # I suspect we will generalize this as we transition to storing all
+                # files in an artifact
+                _, artifact_class = _safe_sdk_import()
+                # we sanitize the key to meet the constraints defined in wandb_artifacts.py
+                # in this case, leaving only alpha numerics or underscores.
+                sanitized_key = re.sub(r"[^a-zA-Z0-9_]+", "", key)
+                art = artifact_class(
+                    "run-{}-{}".format(run.id, sanitized_key), "run_table"
+                )
+                art.add(val, key)
+                run.log_artifact(art)
             val.bind_to_run(run, key, namespace)
         return val.to_json(run)
 
@@ -2073,7 +2157,7 @@ def _wb_filename(
 
 def _numpy_arrays_to_lists(
     payload: Union[dict, Sequence, "np.ndarray"]
-) -> Union[Sequence, dict]:
+) -> Union[Sequence, dict, str, int, float, bool]:
     # Casts all numpy arrays to lists so we don't convert them to histograms, primarily for Plotly
 
     if isinstance(payload, dict):
@@ -2087,7 +2171,9 @@ def _numpy_arrays_to_lists(
         if wandb.TYPE_CHECKING and TYPE_CHECKING:
             payload = cast("np.ndarray", payload)
         return [_numpy_arrays_to_lists(v) for v in payload.tolist()]
-
+    # Protects against logging non serializable objects
+    elif isinstance(payload, Media):
+        return str(payload.__class__.__name__)
     return payload
 
 
