@@ -137,7 +137,12 @@ class MessageRouter(object):
         with self._lock:
             future = self._pending_reqs.pop(msg.uuid, None)
         if future is None:
-            logger.warning("No listener found for msg with uuid %s (%s)", msg.uuid, msg)
+            # TODO (cvp): saw this in tests, seemed benign enough to ignore, but
+            # could point to other issues.
+            if msg.uuid != "":
+                logger.warning(
+                    "No listener found for msg with uuid %s (%s)", msg.uuid, msg
+                )
             return
         future._set_object(msg)
 
@@ -248,6 +253,7 @@ class BackendSender(object):
             proto_artifact.description = artifact.description
         if artifact.metadata:
             proto_artifact.metadata = json.dumps(json_friendly_val(artifact.metadata))  # type: ignore
+        proto_artifact.incremental_beta1 = artifact.incremental
         self._make_artifact_manifest(artifact.manifest, obj=proto_artifact.manifest)
         return proto_artifact
 
@@ -267,7 +273,8 @@ class BackendSender(object):
             proto_entry = proto_manifest.contents.add()
             proto_entry.path = entry.path
             proto_entry.digest = entry.digest
-            proto_entry.size = entry.size
+            if entry.size:
+                proto_entry.size = entry.size
             if entry.birth_artifact_id:
                 proto_entry.birth_artifact_id = entry.birth_artifact_id
             if entry.ref:
@@ -419,7 +426,8 @@ class BackendSender(object):
         get_summary: pb.GetSummaryRequest = None,
         pause: pb.PauseRequest = None,
         resume: pb.ResumeRequest = None,
-        status: pb.StatusRequest = None,
+        stop_status: pb.StopStatusRequest = None,
+        network_status: pb.NetworkStatusRequest = None,
         poll_exit: pb.PollExitRequest = None,
         sampled_history: pb.SampledHistoryRequest = None,
         run_start: pb.RunStartRequest = None,
@@ -436,8 +444,10 @@ class BackendSender(object):
             request.pause.CopyFrom(pause)
         elif resume:
             request.resume.CopyFrom(resume)
-        elif status:
-            request.status.CopyFrom(status)
+        elif stop_status:
+            request.stop_status.CopyFrom(stop_status)
+        elif network_status:
+            request.network_status.CopyFrom(network_status)
         elif poll_exit:
             request.poll_exit.CopyFrom(poll_exit)
         elif sampled_history:
@@ -713,18 +723,29 @@ class BackendSender(object):
         rec = self._make_record(alert=proto_alert)
         self._publish(rec)
 
-    def communicate_status(
-        self, check_stop_req: bool, timeout: int = None
-    ) -> Optional[pb.StatusResponse]:
-        status = pb.StatusRequest()
-        status.check_stop_req = check_stop_req
-        req = self._make_request(status=status)
+    def communicate_stop_status(
+        self, timeout: int = None
+    ) -> Optional[pb.StopStatusResponse]:
+        status = pb.StopStatusRequest()
+        req = self._make_request(stop_status=status)
 
         resp = self._communicate(req, timeout=timeout, local=True)
         if resp is None:
             return None
-        assert resp.response.status_response
-        return resp.response.status_response
+        assert resp.response.stop_status_response
+        return resp.response.stop_status_response
+
+    def communicate_network_status(
+        self, timeout: int = None
+    ) -> Optional[pb.NetworkStatusResponse]:
+        status = pb.NetworkStatusRequest()
+        req = self._make_request(network_status=status)
+
+        resp = self._communicate(req, timeout=timeout, local=True)
+        if resp is None:
+            return None
+        assert resp.response.network_status_response
+        return resp.response.network_status_response
 
     def publish_exit(self, exit_code: int) -> None:
         exit_data = self._make_exit(exit_code)
