@@ -15,7 +15,12 @@ import sys
 from wandb import util
 from wandb.vendor.pynvml import pynvml
 
-from ..lib.filenames import DIFF_FNAME, METADATA_FNAME, REQUIREMENTS_FNAME
+from ..lib.filenames import (
+    CONDA_ENVIRONMENTS_FNAME,
+    DIFF_FNAME,
+    METADATA_FNAME,
+    REQUIREMENTS_FNAME,
+)
 from ..lib.git import GitRepo
 
 if os.name == "posix" and sys.version_info[0] < 3:
@@ -62,9 +67,24 @@ class Meta(object):
             ) as f:
                 f.write("\n".join(installed_packages_list))
         except Exception:
-            logger.error("Error saving pip packages")
+            logger.exception("Error saving pip packages")
         logger.debug("save pip done")
 
+    def _save_conda(self):
+        current_shell_is_conda = os.path.exists(os.path.join(sys.prefix, "conda-meta"))
+        if not current_shell_is_conda:
+            return False
+
+        logger.debug("save conda")
+        try:
+            with open(
+                os.path.join(self._settings.files_dir, CONDA_ENVIRONMENTS_FNAME), "w"
+            ) as f:
+                subprocess.call(["conda", "env", "export"], stdout=f)
+        except Exception:
+            logger.exception("Error saving conda packages")
+        logger.debug("save conda done")
+
     def _save_code(self):
         logger.debug("save code")
         if self._settings.program_relpath is None:
@@ -228,6 +248,7 @@ class Meta(object):
 
         if self._settings._save_requirements:
             self._save_pip()
+            self._save_conda()
         logger.debug("probe done")
 
     def write(self):
