@@ -20,7 +20,7 @@ import shortuuid  # type: ignore
 import six
 import wandb
 from wandb import trigger
-from wandb.errors.error import UsageError
+from wandb.errors import UsageError
 from wandb.integration import sagemaker
 from wandb.integration.magic import magic_install
 from wandb.util import sentry_exc
@@ -69,6 +69,7 @@ class _WandbInit(object):
         self._teardown_hooks = []
         self._wl = None
         self._reporter = None
+        self._use_sagemaker = None
 
     def setup(self, kwargs) -> None:
         """
@@ -102,6 +103,7 @@ class _WandbInit(object):
                 wandb.setup(settings=settings)
             for k, v in six.iteritems(sm_run):
                 kwargs.setdefault(k, v)
+            self._use_sagemaker = True
 
         # Remove parameters that are not part of settings
         init_config = kwargs.pop("config", None) or dict()
@@ -356,7 +358,7 @@ class _WandbInit(object):
             save=drun.save,
             use_artifact=drun.use_artifact,
             log_artifact=drun.log_artifact,
-            define_metric=drun._define_metric,
+            define_metric=drun.define_metric,
             plot_table=drun.plot_table,
             alert=drun.alert,
         )
@@ -448,6 +450,8 @@ class _WandbInit(object):
             if s._windows:
                 tel.env.windows = True
             run._telemetry_imports(tel.imports_init)
+            if self._use_sagemaker:
+                tel.feature.sagemaker = True
 
             if active_start_method == "spawn":
                 tel.env.start_spawn = True
@@ -531,7 +535,7 @@ class _WandbInit(object):
             save=run.save,
             use_artifact=run.use_artifact,
             log_artifact=run.log_artifact,
-            define_metric=run._define_metric,
+            define_metric=run.define_metric,
             plot_table=run.plot_table,
             alert=run.alert,
         )
@@ -647,9 +651,6 @@ def init(
             be stored. When you call download() on an artifact, this is the
             directory where downloaded files will be saved. By default this is
             the ./wandb directory.
-        sync_tensorboard: (bool, optional) Whether to copy all TensorBoard logs
-            to W&B (default: False).
-            [Tensorboard](https://docs.wandb.com/integrations/tensorboard)
         resume (bool, str, optional): Sets the resuming behavior. Options:
             "allow", "must", "never", "auto" or None. Defaults to None.
             Cases:
@@ -699,7 +700,7 @@ def init(
             logged in to W&B. If False, this will let the script run in offline
             mode if a user isn't logged in to W&B. (default: False)
         sync_tensorboard: (bool, optional) Synchronize wandb logs from tensorboard or
-            tensorboardX and saves the relevant events file. Defaults to false.
+            tensorboardX and saves the relevant events file. (default: False)
         monitor_gym: (bool, optional) automatically logs videos of environment when
             using OpenAI Gym. (default: False)
             See https://docs.wandb.com/library/integrations/openai-gym
