@@ -32,6 +32,7 @@ import json
 import multiprocessing
 import os
 import platform
+import re
 import socket
 import sys
 import tempfile
@@ -45,20 +46,30 @@ from .lib.git import GitRepo
 from .lib.ipython import _get_python_type
 from .lib.runid import generate_id
 
-if wandb.TYPE_CHECKING:  # type: ignore
+if wandb.TYPE_CHECKING:
+    from wandb.sdk.wandb_config import Config
+    from wandb.sdk.wandb_setup import _EarlyLogger
     from typing import (  # noqa: F401 pylint: disable=unused-import
+        cast,
+        Any,
         Dict,
         List,
-        Optional,
-        Union,
-        Tuple,
-        Callable,
         Set,
-        Type,
+        Callable,
+        Generator,
+        Iterable,
+        Iterator,
+        Optional,
         Sequence,
+        Tuple,
+        Type,
+        Union,
+        TYPE_CHECKING,
     )
 
-defaults: Dict[str, Union[str, int, Tuple]] = dict(
+    Defaults = Dict[str, Union[str, int, bool, Tuple]]
+
+defaults: Defaults = dict(
     base_url="https://api.wandb.ai",
     summary_warnings=5,
     git_remote="origin",
@@ -92,6 +103,7 @@ env_settings: Dict[str, Optional[str]] = dict(
     silent=None,
     sagemaker_disable=None,
     start_method=None,
+    strict=None,
     root_dir="WANDB_DIR",
     run_name="WANDB_NAME",
     run_notes="WANDB_NOTES",
@@ -100,11 +112,11 @@ env_settings: Dict[str, Optional[str]] = dict(
 )
 
 env_convert: Dict[str, Callable[[str], List[str]]] = dict(
-    run_tags=lambda s: s.split(","), ignore_globs=lambda s: s.split(","),
+    run_tags=lambda s: s.split(","), ignore_globs=lambda s: s.split(",")
 )
 
 
-def _build_inverse_map(prefix: str, d: Dict[str, str]) -> Dict[str, str]:
+def _build_inverse_map(prefix: str, d: Dict[str, Optional[str]]) -> Dict[str, str]:
     inv_map = dict()
     for k, v in six.iteritems(d):
         v = v or prefix + k.upper()
@@ -116,7 +128,7 @@ def _error_choices(value: str, choices: Set[str]) -> str:
     return "{} not in {}".format(value, ",".join(list(choices)))
 
 
-def _get_program():
+def _get_program() -> Optional[Any]:
     program = os.getenv(wandb.env.PROGRAM)
     if program:
         return program
@@ -129,7 +141,9 @@ def _get_program():
         return None
 
 
-def _get_program_relpath_from_gitrepo(program, _logger=None):
+def _get_program_relpath_from_gitrepo(
+    program: str, _logger: Optional[_EarlyLogger] = None
+) -> Optional[str]:
     repo = GitRepo()
     root = repo.root
     if not root:
@@ -153,7 +167,7 @@ def _get_program_relpath_from_gitrepo(program, _logger=None):
 # the setting exposed to users as `dir=` or `WANDB_DIR` is actually
 # the `root_dir`. We add the `__stage_dir__` to it to get the full
 # `wandb_dir`
-def get_wandb_dir(root_dir: str):
+def get_wandb_dir(root_dir: str) -> str:
     # We use the hidden version if it already exists, otherwise non-hidden.
     if os.path.exists(os.path.join(root_dir, ".wandb")):
         __stage_dir__ = ".wandb" + os.sep
@@ -168,12 +182,13 @@ def get_wandb_dir(root_dir: str):
     return path
 
 
-def _str_as_bool(val) -> Optional[bool]:
+def _str_as_bool(val: str) -> Optional[bool]:
+    ret_val = None
     try:
-        val = bool(strtobool(val))
+        ret_val = bool(strtobool(val))
     except (AttributeError, ValueError):
         pass
-    return val if isinstance(val, bool) else None
+    return ret_val
 
 
 @enum.unique
@@ -199,7 +214,8 @@ class Settings(object):
     console: str = "auto"
     disabled: bool = False
     run_tags: Optional[Tuple] = None
-
+    run_id: Optional[str] = None
+    sweep_id: Optional[str] = None
     resume_fname_spec: Optional[str] = None
     root_dir: Optional[str] = None
     log_dir_spec: Optional[str] = None
@@ -214,15 +230,20 @@ class Settings(object):
     sync_symlink_latest_spec: Optional[str] = None
     settings_system_spec: Optional[str] = None
     settings_workspace_spec: Optional[str] = None
-    silent: bool = False
-    show_info: bool = True
-    show_warnings: bool = True
-    show_errors: bool = True
+    silent: str = "False"
+    show_info: str = "True"
+    show_warnings: str = "True"
+    show_errors: str = "True"
+    username: Optional[str]
     email: Optional[str] = None
     save_code: Optional[bool] = None
     code_dir: Optional[str] = None
     program_relpath: Optional[str] = None
+    program: Optional[str]
+    notebook_name: Optional[str]
     host: Optional[str]
+    resume: str
+    strict: Optional[str] = None
 
     # Public attributes
     entity: Optional[str] = None
@@ -237,8 +258,10 @@ class Settings(object):
     base_url: Optional[str] = None
 
     # Private attributes
-    __start_time: Optional[float]
-    __start_datetime: Optional[datetime]
+    _start_time: Optional[float]
+    _start_datetime: Optional[datetime]
+    _unsaved_keys: List[str]
+    _except_exit: Optional[bool]
 
     # Internal attributes
     __frozen: bool
@@ -269,7 +292,7 @@ class Settings(object):
         self,
         base_url: str = None,
         api_key: str = None,
-        anonymous=None,
+        anonymous: str = None,
         mode: str = None,
         start_method: str = None,
         entity: str = None,
@@ -282,39 +305,39 @@ class Settings(object):
         resume: str = None,
         magic: Union[Dict, str, bool] = False,
         run_tags: Sequence = None,
-        sweep_id=None,
+        sweep_id: str = None,
         allow_val_change: bool = None,
         force: bool = None,
         relogin: bool = None,
         # compatibility / error handling
         # compat_version=None,  # set to "0.8" for safer defaults for older users
-        # strict=None,  # set to "on" to enforce current best practices (also "warn")
-        problem="fatal",
+        strict: str = None,
+        problem: str = "fatal",
         # dynamic settings
-        system_sample_seconds=2,
-        system_samples=15,
-        heartbeat_seconds=30,
-        config_paths=None,
-        sweep_param_path=None,
-        _config_dict=None,
+        system_sample_seconds: int = 2,
+        system_samples: int = 15,
+        heartbeat_seconds: int = 30,
+        config_paths: List[str] = None,
+        sweep_param_path: str = None,
+        _config_dict: Config = None,
         # directories and files
-        root_dir=None,
-        settings_system_spec="~/.config/wandb/settings",
-        settings_workspace_spec="{wandb_dir}/settings",
-        sync_dir_spec="{wandb_dir}/{run_mode}-{timespec}-{run_id}",
-        sync_file_spec="run-{run_id}.wandb",
+        root_dir: str = None,
+        settings_system_spec: str = "~/.config/wandb/settings",
+        settings_workspace_spec: str = "{wandb_dir}/settings",
+        sync_dir_spec: str = "{wandb_dir}/{run_mode}-{timespec}-{run_id}",
+        sync_file_spec: str = "run-{run_id}.wandb",
         # sync_symlink_sync_spec="{wandb_dir}/sync",
         # sync_symlink_offline_spec="{wandb_dir}/offline",
-        sync_symlink_latest_spec="{wandb_dir}/latest-run",
-        log_dir_spec="{wandb_dir}/{run_mode}-{timespec}-{run_id}/logs",
-        log_user_spec="debug.log",
-        log_internal_spec="debug-internal.log",
-        log_symlink_user_spec="{wandb_dir}/debug.log",
-        log_symlink_internal_spec="{wandb_dir}/debug-internal.log",
-        resume_fname_spec="{wandb_dir}/wandb-resume.json",
-        files_dir_spec="{wandb_dir}/{run_mode}-{timespec}-{run_id}/files",
-        tmp_dir_spec="{wandb_dir}/{run_mode}-{timespec}-{run_id}/tmp",
-        symlink=None,  # probed
+        sync_symlink_latest_spec: str = "{wandb_dir}/latest-run",
+        log_dir_spec: str = "{wandb_dir}/{run_mode}-{timespec}-{run_id}/logs",
+        log_user_spec: str = "debug.log",
+        log_internal_spec: str = "debug-internal.log",
+        log_symlink_user_spec: str = "{wandb_dir}/debug.log",
+        log_symlink_internal_spec: str = "{wandb_dir}/debug-internal.log",
+        resume_fname_spec: str = "{wandb_dir}/wandb-resume.json",
+        files_dir_spec: str = "{wandb_dir}/{run_mode}-{timespec}-{run_id}/files",
+        tmp_dir_spec: str = "{wandb_dir}/{run_mode}-{timespec}-{run_id}/tmp",
+        symlink: bool = None,  # probed
         # where files are temporary stored when saving
         # files_dir=None,
         # tmp_dir=None,
@@ -323,51 +346,51 @@ class Settings(object):
         # data_spec="wandb-{timespec}-{pid}-data.bin",
         # run_base_dir="wandb",
         # run_dir_spec="run-{timespec}-{pid}",
-        program=None,
-        notebook_name=None,
-        disable_code=None,
-        ignore_globs=None,
-        save_code=None,
-        code_dir=None,
-        program_relpath=None,
-        git_remote=None,
-        dev_prod=None,  # in old settings files, TODO: support?
-        host=None,
-        username=None,
-        email=None,
-        docker=None,
-        sagemaker_disable: Optional[bool] = None,
-        _start_time=None,
-        _start_datetime=None,
-        _cli_only_mode=None,  # avoid running any code specific for runs
-        _disable_viewer=None,  # prevent early viewer query
-        console=None,
-        disabled=None,  # alias for mode=dryrun, not supported yet
-        reinit=None,
-        _save_requirements=True,
+        program: str = None,
+        notebook_name: str = None,
+        disable_code: bool = None,
+        ignore_globs: bool = None,
+        save_code: bool = None,
+        code_dir: str = None,
+        program_relpath: str = None,
+        git_remote: str = None,
+        # dev_prod=None,  # in old settings files, TODO: support?
+        host: str = None,
+        username: str = None,
+        email: str = None,
+        docker: str = None,
+        sagemaker_disable: bool = None,
+        _start_time: float = None,
+        _start_datetime: datetime = None,
+        _cli_only_mode: bool = None,  # avoid running any code specific for runs
+        _disable_viewer: bool = None,  # prevent early viewer query
+        console: str = None,
+        disabled: bool = None,  # alias for mode=dryrun, not supported yet
+        reinit: bool = None,
+        _save_requirements: bool = True,
         # compute environment
-        show_colors=None,
-        show_emoji=None,
-        silent=None,
-        show_info=None,
-        show_warnings=None,
-        show_errors=None,
-        summary_errors=None,
-        summary_warnings=None,
-        _internal_queue_timeout=2,
-        _internal_check_process=8,
-        _disable_meta=None,
-        _disable_stats=None,
-        _jupyter_path=None,
-        _jupyter_name=None,
-        _jupyter_root=None,
-        _executable=None,
-        _cuda=None,
-        _args=None,
-        _os=None,
-        _python=None,
-        _kaggle=None,
-        _except_exit=None,
+        show_colors: bool = None,
+        show_emoji: bool = None,
+        silent: bool = None,
+        show_info: bool = None,
+        show_warnings: bool = None,
+        show_errors: bool = None,
+        summary_errors: int = None,
+        summary_warnings: int = None,
+        _internal_queue_timeout: float = 2,
+        _internal_check_process: float = 8,
+        _disable_meta: bool = None,
+        _disable_stats: bool = None,
+        _jupyter_path: str = None,
+        _jupyter_name: str = None,
+        _jupyter_root: str = None,
+        _executable: str = None,
+        _cuda: bool = None,
+        _args: List[Any] = None,
+        _os: str = None,
+        _python: str = None,
+        _kaggle: str = None,
+        _except_exit: str = None,
     ):
         kwargs = dict(locals())
         kwargs.pop("self")
@@ -379,8 +402,8 @@ class Settings(object):
         object.__setattr__(self, "_Settings__override_dict", dict())
         object.__setattr__(self, "_Settings__defaults_dict_set", dict())
         object.__setattr__(self, "_Settings__override_dict_set", dict())
-        object.__setattr__(self, "_Settings__start_datetime", None)
-        object.__setattr__(self, "_Settings__start_time", None)
+        object.__setattr__(self, "_Settings_start_datetime", None)
+        object.__setattr__(self, "_Settings_start_time", None)
         class_defaults = self._get_class_defaults()
         self._apply_defaults(class_defaults)
         self._apply_defaults(defaults)
@@ -403,6 +426,12 @@ class Settings(object):
             return None
         return _str_as_bool(self.silent)
 
+    @property
+    def _strict(self) -> Optional[bool]:
+        if not self.strict:
+            return None
+        return _str_as_bool(self.strict)
+
     @property
     def _show_info(self) -> Optional[bool]:
         if not self.show_info:
@@ -427,11 +456,14 @@ class Settings(object):
 
     @property
     def _jupyter(self) -> bool:
-        return _get_python_type() != "python"
+        return str(_get_python_type()) != "python"
 
     @property
     def _kaggle(self) -> bool:
-        return util._is_kaggle()
+        is_kaggle = util._is_likely_kaggle()
+        if wandb.TYPE_CHECKING and TYPE_CHECKING:
+            assert isinstance(is_kaggle, bool)
+        return is_kaggle
 
     @property
     def _windows(self) -> bool:
@@ -480,31 +512,37 @@ class Settings(object):
 
     @property
     def resume_fname(self) -> str:
-        return self._path_convert(self.resume_fname_spec)
+        resume_fname = self._path_convert(self.resume_fname_spec)
+        if wandb.TYPE_CHECKING and TYPE_CHECKING:
+            assert isinstance(resume_fname, str)
+        return resume_fname
 
     @property
     def wandb_dir(self) -> str:
         return get_wandb_dir(self.root_dir or "")
 
     @property
-    def log_user(self) -> str:
+    def log_user(self) -> Optional[str]:
         return self._path_convert(self.log_dir_spec, self.log_user_spec)
 
     @property
-    def log_internal(self) -> str:
+    def log_internal(self) -> Optional[str]:
         return self._path_convert(self.log_dir_spec, self.log_internal_spec)
 
     @property
-    def _sync_dir(self) -> str:
+    def _sync_dir(self) -> Optional[str]:
         return self._path_convert(self.sync_dir_spec)
 
     @property
-    def sync_file(self) -> str:
+    def sync_file(self) -> Optional[str]:
         return self._path_convert(self.sync_dir_spec, self.sync_file_spec)
 
     @property
     def files_dir(self) -> str:
-        return self._path_convert(self.files_dir_spec)
+        file_path = self._path_convert(self.files_dir_spec)
+        if wandb.TYPE_CHECKING and TYPE_CHECKING:
+            assert isinstance(file_path, str)
+        return file_path
 
     @property
     def tmp_dir(self) -> str:
@@ -515,116 +553,133 @@ class Settings(object):
         return os.path.join(self.tmp_dir, "code")
 
     @property
-    def log_symlink_user(self) -> str:
+    def log_symlink_user(self) -> Optional[str]:
         return self._path_convert(self.log_symlink_user_spec)
 
     @property
-    def log_symlink_internal(self) -> str:
+    def log_symlink_internal(self) -> Optional[str]:
         return self._path_convert(self.log_symlink_internal_spec)
 
     @property
-    def sync_symlink_latest(self) -> str:
+    def sync_symlink_latest(self) -> Optional[str]:
         return self._path_convert(self.sync_symlink_latest_spec)
 
     @property
-    def settings_system(self) -> str:
+    def settings_system(self) -> Optional[str]:
         return self._path_convert(self.settings_system_spec)
 
     @property
-    def settings_workspace(self) -> str:
+    def settings_workspace(self) -> Optional[str]:
         return self._path_convert(self.settings_workspace_spec)
 
-    def _validate_start_method(self, value):
+    def _validate_start_method(self, value: str) -> Optional[str]:
         available_methods = ["thread"]
         if hasattr(multiprocessing, "get_all_start_methods"):
             available_methods += multiprocessing.get_all_start_methods()
         if value in available_methods:
-            return
-        return _error_choices(value, available_methods)
+            return None
+        return _error_choices(value, set(available_methods))
 
-    def _validate_mode(self, value):
-        choices = {
-            "dryrun",
-            "run",
-            "offline",
-            "online",
-            "disabled",
-        }
+    def _validate_mode(self, value: str) -> Optional[str]:
+        choices = {"dryrun", "run", "offline", "online", "disabled"}
         if value in choices:
-            return
+            return None
         return _error_choices(value, choices)
 
-    def _validate_console(self, value):
+    def _validate_console(self, value: str) -> Optional[str]:
         # choices = {"auto", "redirect", "off", "file", "iowrap", "notebook"}
-        choices = {
-            "auto",
-            "redirect",
-            "off",
-            "wrap",
-        }
+        choices = {"auto", "redirect", "off", "wrap"}
         if value in choices:
-            return
+            return None
         return _error_choices(value, choices)
 
-    def _validate_problem(self, value):
+    def _validate_problem(self, value: str) -> Optional[str]:
         choices = {"fatal", "warn", "silent"}
         if value in choices:
-            return
+            return None
         return _error_choices(value, choices)
 
-    def _validate_anonymous(self, value):
+    def _validate_anonymous(self, value: str) -> Optional[str]:
         choices = {"allow", "must", "never", "false", "true"}
         if value in choices:
-            return
+            return None
         return _error_choices(value, choices)
 
-    def _validate_silent(self, value):
+    def _validate_strict(self, value: str) -> Optional[str]:
         val = _str_as_bool(value)
         if val is None:
             return "{} is not a boolean".format(value)
+        return None
 
-    def _validate_show_info(self, value):
+    def _validate_silent(self, value: str) -> Optional[str]:
         val = _str_as_bool(value)
         if val is None:
             return "{} is not a boolean".format(value)
+        return None
 
-    def _validate_show_warnings(self, value):
+    def _validate_show_info(self, value: str) -> Optional[str]:
         val = _str_as_bool(value)
         if val is None:
             return "{} is not a boolean".format(value)
+        return None
 
-    def _validate_show_errors(self, value):
+    def _validate_show_warnings(self, value: str) -> Optional[str]:
         val = _str_as_bool(value)
         if val is None:
             return "{} is not a boolean".format(value)
+        return None
 
-    def _preprocess_base_url(self, value):
+    def _validate_show_errors(self, value: str) -> Optional[str]:
+        val = _str_as_bool(value)
+        if val is None:
+            return "{} is not a boolean".format(value)
+        return None
+
+    def _validate_base_url(self, value: Optional[str]) -> Optional[str]:
+        if value is not None:
+            if re.match(r".*wandb\.ai[^\.]*$", value) and "api." not in value:
+                # user might guess app.wandb.ai or wandb.ai is the default cloud server
+                return "{} is not a valid server address, did you mean https://api.wandb.ai?".format(
+                    value
+                )
+            elif re.match(r".*wandb\.ai[^\.]*$", value) and "http://" in value:
+                return "http is not secure, please use https://api.wandb.ai"
+        return None
+
+    def _preprocess_base_url(self, value: Optional[str]) -> Optional[str]:
         if value is not None:
             value = value.rstrip("/")
         return value
 
-    def _start_run(self):
+    def _start_run(self) -> None:
         datetime_now: datetime = datetime.now()
         time_now: float = time.time()
-        object.__setattr__(self, "_Settings__start_datetime", datetime_now)
-        object.__setattr__(self, "_Settings__start_time", time_now)
+        object.__setattr__(self, "_Settings_start_datetime", datetime_now)
+        object.__setattr__(self, "_Settings_start_time", time_now)
 
-    def _apply_settings(self, settings, _logger=None):
+    def _apply_settings(
+        self, settings: "Settings", _logger: Optional[_EarlyLogger] = None
+    ) -> None:
         # TODO(jhr): make a more efficient version of this
         for k in settings._public_keys():
             source = settings.__defaults_dict.get(k)
             self._update({k: settings[k]}, _source=source)
 
-    def _apply_defaults(self, defaults):
+    def _apply_defaults(self, defaults: Defaults) -> None:
         self._update(defaults, _source=self.Source.BASE)
 
-    def _apply_configfiles(self, _logger=None):
+    def _apply_configfiles(self, _logger: Optional[_EarlyLogger] = None) -> None:
         # TODO(jhr): permit setting of config in system and workspace
-        self._update(self._load(self.settings_system), _source=self.Source.SYSTEM)
+        if self.settings_system is not None:
+            self._update(self._load(self.settings_system), _source=self.Source.SYSTEM)
+        if self.settings_workspace is not None:
+            self._update(
+                self._load(self.settings_workspace), _source=self.Source.WORKSPACE
+            )
 
-        self._update(self._load(self.settings_workspace), _source=self.Source.WORKSPACE)
-
-    def _apply_environ(self, environ, _logger=None):
+    def _apply_environ(
+        self, environ: os._Environ, _logger: Optional[_EarlyLogger] = None
+    ) -> None:
         inv_map = _build_inverse_map(env_prefix, env_settings)
         env_dict = dict()
         for k, v in six.iteritems(environ):
@@ -644,17 +699,23 @@ class Settings(object):
             _logger.info("setting env: {}".format(env_dict))
         self._update(env_dict, _source=self.Source.ENV)
 
-    def _apply_user(self, user_settings, _logger=None):
+    def _apply_user(
+        self, user_settings: Dict[str, Any], _logger: Optional[_EarlyLogger] = None
+    ) -> None:
         if _logger:
             _logger.info("setting user settings: {}".format(user_settings))
         self._update(user_settings, _source=self.Source.USER)
 
-    def _apply_source_login(self, login_settings, _logger=None):
+    def _apply_source_login(
+        self, login_settings: Dict[str, Any], _logger: Optional[_EarlyLogger] = None
+    ) -> None:
         if _logger:
             _logger.info("setting login settings: {}".format(login_settings))
         self._update(login_settings, _source=self.Source.LOGIN)
 
-    def _path_convert_part(self, path_part, format_dict):
+    def _path_convert_part(
+        self, path_part: str, format_dict: Dict[str, Any]
+    ) -> Optional[List[str]]:
         """convert slashes, expand ~ and other macros."""
 
         path_parts = path_part.split(os.sep if os.sep in path_part else "/")
@@ -665,11 +726,11 @@ class Settings(object):
                 return None
         return path_parts
 
-    def _path_convert(self, *path):
+    def _path_convert(self, *path: Any) -> Optional[str]:
         """convert slashes, expand ~ and other macros."""
 
-        format_dict = dict()
-        if self._start_time:
+        format_dict: Dict[str, Union[str, int]] = dict()
+        if self._start_time and self._start_datetime:
             format_dict["timespec"] = datetime.strftime(
                 self._start_datetime, "%Y%m%d_%H%M%S"
             )
@@ -681,35 +742,35 @@ class Settings(object):
         #              this is wrong if the run_dir changes later
         format_dict["wandb_dir"] = self.wandb_dir or "wandb"
 
-        path_items = []
+        path_items: List[str] = []
         for p in path:
             part = self._path_convert_part(p, format_dict)
             if part is None:
                 return None
             path_items += part
-        path = os.path.join(*path_items)
-        path = os.path.expanduser(path)
-        return path
+        converted_path = os.path.join(*path_items)
+        converted_path = os.path.expanduser(converted_path)
+        return converted_path
 
-    # def _clear_early_logger(self):
+    # def _clear_early_logger(self) -> None:
     #     # TODO(jhr): this is a hack
     #     object.__setattr__(self, "_Settings__early_logger", None)
 
-    def _setup(self, kwargs):
+    def _setup(self, kwargs: Any) -> None:
         for k, v in six.iteritems(kwargs):
             if k not in self._unsaved_keys:
                 object.__setattr__(self, k, v)
 
-    def __copy__(self):
+    def __copy__(self) -> "Settings":
         """Copy (note that the copied object will not be frozen)."""
         s = Settings()
         s._apply_settings(self)
         return s
 
-    def duplicate(self):
+    def duplicate(self) -> "Settings":
         return copy.copy(self)
 
-    def _check_invalid(self, k, v):
+    def _check_invalid(self, k: str, v: Any) -> None:
         if v is None:
             return
         f = getattr(self, "_validate_" + k, None)
@@ -719,14 +780,20 @@ class Settings(object):
         if invalid:
             raise TypeError("Settings field {}: {}".format(k, invalid))
 
-    def _perform_preprocess(self, k, v):
+    def _perform_preprocess(self, k: str, v: Any) -> Optional[Any]:
         f = getattr(self, "_preprocess_" + k, None)
         if not f or not callable(f):
             return v
         else:
             return f(v)
 
-    def _update(self, __d=None, _source=None, _override=None, **kwargs):
+    def _update(
+        self,
+        __d: Dict[str, Any] = None,
+        _source: Optional[int] = None,
+        _override: Optional[int] = None,
+        **kwargs: Any
+    ) -> None:
         if self.__frozen and (__d or kwargs):
             raise TypeError("Settings object is frozen")
         d = __d or dict()
@@ -753,8 +820,16 @@ class Settings(object):
                 self.__override_dict[k] = _override
                 self.__override_dict_set.setdefault(k, set()).add(_override)
 
-    def update(self, __d=None, **kwargs):
-        self._update(__d, **kwargs)
+    def update(self, __d: Dict = None, **kwargs: Any) -> None:
+        _source = kwargs.pop("_source", None)
+        _override = kwargs.pop("_override", None)
+        if wandb.TYPE_CHECKING and TYPE_CHECKING:
+            _source = cast(Optional[int], _source)
+            _override = cast(Optional[int], _override)
+
+        self._update(__d, _source=_source, _override=_override, **kwargs)
+
+        # self._update(__d, **kwargs)
 
     def _priority_failed(
         self, k: str, source: Optional[int], override: Optional[int]
@@ -771,10 +846,10 @@ class Settings(object):
             return True
         return False
 
-    def _infer_settings_from_env(self):
+    def _infer_settings_from_env(self) -> None:
         """Modify settings based on environment (for runs and cli)."""
 
-        d = {}
+        d: Defaults = {}
         # disable symlinks if on windows (requires admin or developer setup)
         d["symlink"] = True
         if self._windows:
@@ -799,7 +874,11 @@ class Settings(object):
             u["_jupyter_path"] = meta.get("path")
             u["_jupyter_name"] = meta.get("name")
             u["_jupyter_root"] = meta.get("root")
-        elif self._jupyter and os.path.exists(self.notebook_name):
+        elif (
+            self._jupyter
+            and self.notebook_name is not None
+            and os.path.exists(self.notebook_name)
+        ):
             u["_jupyter_path"] = self.notebook_name
             u["_jupyter_name"] = self.notebook_name
             u["_jupyter_root"] = os.getcwd()
@@ -840,7 +919,9 @@ class Settings(object):
 
         self.update(u)
 
-    def _infer_run_settings_from_env(self, _logger=None):
+    def _infer_run_settings_from_env(
+        self, _logger: Optional[_EarlyLogger] = None
+    ) -> None:
         """Modify settings based on environment (for runs only)."""
         # If there's not already a program file, infer it now.
         program = self.program or _get_program()
@@ -853,7 +934,7 @@ class Settings(object):
             program = "<python with no main file>"
             self.update(dict(program=program))
 
-    def setdefaults(self, __d=None):
+    def setdefaults(self, __d: Optional[Defaults] = None) -> None:
         __d = __d or defaults
         # set defaults
         for k, v in __d.items():
@@ -861,13 +942,13 @@ class Settings(object):
                 if self.__dict__.get(k) is None:
                     object.__setattr__(self, k, v)
 
-    def save(self, fname):
+    def save(self, fname: str) -> None:
         pass
 
-    def load(self, fname):
+    def load(self, fname: str) -> None:
         pass
 
-    def __setattr__(self, name, value):
+    def __setattr__(self, name: str, value: Any) -> None:
         if name not in self.__dict__:
             raise AttributeError(name)
         if self.__frozen:
@@ -877,11 +958,11 @@ class Settings(object):
         object.__setattr__(self, name, value)
 
     @classmethod
-    def _property_keys(cls):
+    def _property_keys(cls) -> Generator[str, None, None]:
         return (k for k, v in vars(cls).items() if isinstance(v, property))
 
     @classmethod
-    def _class_keys(cls):
+    def _class_keys(cls) -> Generator[str, None, None]:
         return (
             k
             for k, v in vars(cls).items()
@@ -889,37 +970,37 @@ class Settings(object):
         )
 
     @classmethod
-    def _get_class_defaults(cls):
+    def _get_class_defaults(cls) -> dict:
         class_keys = set(cls._class_keys())
         return dict(
             (k, v) for k, v in vars(cls).items() if k in class_keys and v is not None
         )
 
-    def _public_keys(self):
+    def _public_keys(self) -> Iterator[str]:
         return filter(lambda x: not x.startswith("_Settings__"), self.__dict__)
 
-    def keys(self):
+    def keys(self) -> Iterable[str]:
         return itertools.chain(self._public_keys(), self._property_keys())
 
-    def __getitem__(self, k):
+    def __getitem__(self, k: str) -> Any:
         props = self._property_keys()
         if k in props:
             return getattr(self, k)
         return self.__dict__[k]
 
-    def freeze(self):
+    def freeze(self) -> "Settings":
         self.__frozen = True
         return self
 
     def is_frozen(self) -> bool:
         return self.__frozen
 
-    def _load(self, fname):
+    def _load(self, fname: str) -> dict:
         section = "default"
         cp = configparser.ConfigParser()
         cp.add_section(section)
         cp.read(fname)
-        d = dict()
+        d: Dict[str, Any] = dict()
         for k in cp[section]:
             d[k] = cp[section][k]
             # TODO (cvp): we didn't do this in the old cli, but it seems necessary
@@ -927,18 +1008,20 @@ class Settings(object):
                 d[k] = d[k].split(",")
         return d
 
-    def _apply_login(self, args, _logger=None):
-        param_map = dict(key="api_key", host="base_url",)
+    def _apply_login(
+        self, args: Dict[str, Any], _logger: Optional[_EarlyLogger] = None
+    ) -> None:
+        param_map = dict(key="api_key", host="base_url")
         args = {param_map.get(k, k): v for k, v in six.iteritems(args) if v is not None}
         self._apply_source_login(args, _logger=_logger)
 
-    def _apply_init_login(self, args):
+    def _apply_init_login(self, args: Dict[str, Any]) -> None:
         # apply some init parameters dealing with login
         keys = {"mode"}
         args = {k: v for k, v in six.iteritems(args) if k in keys and v is not None}
         self._update(args, _source=self.Source.INIT)
 
-    def _apply_init(self, args):
+    def _apply_init(self, args: Dict[str, Union[str, int, None]]) -> None:
         # prevent setting project, entity if in sweep
         # TODO(jhr): these should be locked elements in the future
         if self.sweep_id:
@@ -988,29 +1071,39 @@ class Settings(object):
                     )
         self.run_id = self.run_id or generate_id()
         # persist our run id incase of failure
-        if self.resume == "auto":
+        # check None for mypy
+        if self.resume == "auto" and self.resume_fname is not None:
             wandb.util.mkdir_exists_ok(self.wandb_dir)
             with open(self.resume_fname, "w") as f:
                 f.write(json.dumps({"run_id": self.run_id}))
 
-    def _as_source(self, source, override=None):
+    def _as_source(
+        self, source: int, override: Optional[int] = None
+    ) -> "Settings._Setter":
         return Settings._Setter(settings=self, source=source, override=override)
 
     class _Setter(object):
-        def __init__(self, settings, source, override):
+        _settings: "Settings"
+        _source: int
+        _override: int
+
+        def __init__(
+            self, settings: "Settings", source: int, override: Optional[int]
+        ) -> None:
             object.__setattr__(self, "_settings", settings)
             object.__setattr__(self, "_source", source)
             object.__setattr__(self, "_override", override)
 
-        def __enter__(self):
+        def __enter__(self) -> "Settings._Setter":
             return self
 
-        def __exit__(self, exc_type, exc_value, exc_traceback):
+        def __exit__(self, exc_type: Any, exc_value: Any, exc_traceback: Any) -> None:
             pass
 
-        def __setattr__(self, name, value):
+        def __setattr__(self, name: str, value: Any) -> None:
             self.update({name: value})
 
-        def update(self, *args, **kwargs):
-            kwargs.update(_source=self._source, _override=self._override)
-            self._settings.update(*args, **kwargs)
+        def update(self, *args: Dict, **kwargs: str) -> None:
+            self._settings.update(
+                *args, _source=self._source, _override=self._override, **kwargs
+            )
