@@ -1,7 +1,8 @@
+import math
 import sys
 import typing as t
 
-from wandb.util import get_module
+from wandb.util import get_module, is_numpy_array
 
 np = get_module("numpy")  # intentionally not required
 
@@ -45,6 +46,15 @@ class TypeRegistry:
 
     @staticmethod
     def type_of(py_obj):
+        # Special case handler for common case of np.nans. np.nan
+        # is of type 'float', but should be treated as a None. This is
+        # because np.nan can co-exist with other types in dataframes,
+        # but will be ultimately treated as a None. Ignoring type since
+        # mypy does not trust that py_obj is a float by the time it is
+        # passed to isnan.
+        if py_obj.__class__ == float and math.isnan(py_obj):  # type: ignore
+            return NoneType()
+
         class_handler = TypeRegistry.types_by_class().get(py_obj.__class__)
         _type = None
         if class_handler:
@@ -274,7 +284,8 @@ class Type(object):
         return self is other or (
             isinstance(self, Type)
             and isinstance(other, Type)
-            and self.to_json() == other.to_json()
+            and self.params.keys() == other.params.keys()
+            and all([self.params[k] == other.params[k] for k in self.params])
         )
 
 
@@ -562,13 +573,17 @@ class ListType(Type):
     name = "list"
     types = [list, tuple, set, frozenset]
 
-    def __init__(self, element_type = None):
+    def __init__(
+        self,
+        element_type = None,
+        length = None,
+    ):
         if element_type is None:
             wb_type = UnknownType()
         else:
             wb_type = TypeRegistry.type_from_dtype(element_type)
 
-        self.params.update({"element_type": wb_type})
+        self.params.update({"element_type": wb_type, "length": length})
 
     @classmethod
     def from_obj(cls, py_obj = None):
@@ -595,7 +610,7 @@ class ListType(Type):
 
                 elm_type = _elm_type
 
-            return cls(elm_type)
+            return cls(elm_type, len(py_list))
 
     def assign_type(self, wb_type):
         if isinstance(wb_type, ListType):
@@ -603,7 +618,12 @@ class ListType(Type):
                 wb_type.params["element_type"]
             )
             if not isinstance(assigned_type, InvalidType):
-                return ListType(assigned_type)
+                return ListType(
+                    assigned_type,
+                    None
+                    if self.params["length"] != wb_type.params["length"]
+                    else self.params["length"],
+                )
 
         return InvalidType()
 
@@ -614,11 +634,12 @@ class ListType(Type):
             new_element_type = self.params["element_type"]
             # The following ignore is needed since the above hasattr(py_obj, "__iter__") enforces iteration
             # error: Argument 1 to "list" has incompatible type "Optional[Any]"; expected "Iterable[Any]"
-            for obj in list(py_obj):  # type: ignore
+            py_list = list(py_obj)  # type: ignore
+            for obj in py_list:
                 new_element_type = new_element_type.assign(obj)
                 if isinstance(new_element_type, InvalidType):
                     return InvalidType()
-            return ListType(new_element_type)
+            return ListType(new_element_type, len(py_list))
 
         return InvalidType()
 
@@ -646,8 +667,89 @@ class ListType(Type):
         return "{}[]".format(self.params["element_type"])
 
 
+class NDArrayType(Type):
+    """Represents a list of homogenous types
+    """
+
+    name = "ndarray"
+    types = []  # will manually add type if np is available
+    # _serialization_path: t.Optional[t.Dict[str, str]]
+
+    def __init__(
+        self,
+        shape,
+        serialization_path = None,
+    ):
+        self.params.update({"shape": list(shape)})
+        self._serialization_path = serialization_path
+
+    @classmethod
+    def from_obj(cls, py_obj = None):
+        if is_numpy_array(py_obj):
+            return cls(py_obj.shape)  # type: ignore
+        elif isinstance(py_obj, list):
+            shape = []
+            target = py_obj
+            while isinstance(target, list):
+                dim = len(target)
+                shape.append(dim)
+                if dim > 0:
+                    target = target[0]
+            return cls(shape)
+        else:
+            raise TypeError(
+                "NDArrayType.from_obj expects py_obj to be ndarray or list, found {}".format(
+                    py_obj.__class__
+                )
+            )
+
+    def assign_type(self, wb_type):
+        if (
+            isinstance(wb_type, NDArrayType)
+            and self.params["shape"] == wb_type.params["shape"]
+        ):
+            return self
+        elif isinstance(wb_type, ListType):
+            # Should we return error here?
+            return self
+
+        return InvalidType()
+
+    def assign(
+        self, py_obj = None
+    ):
+        if is_numpy_array(py_obj) or isinstance(py_obj, list):
+            py_type = self.from_obj(py_obj)
+            return self.assign_type(py_type)
+
+        return InvalidType()
+
+    def to_json(
+        self, artifact = None
+    ):
+        # custom override to support serialization path outside of params internal dict
+        res = {
+            "wb_type": self.name,
+            "params": {
+                "shape": self.params["shape"],
+                "serialization_path": self._serialization_path,
+            },
+        }
+
+        return res
+
+    def _get_serialization_path(self):
+        return self._serialization_path
+
+    def _set_serialization_path(self, path, key):
+        self._serialization_path = {"path": path, "key": key}
+
+    def _clear_serialization_path(self):
+        self._serialization_path = None
+
+
 if np:
-    ListType.types.append(np.ndarray)
+    NDArrayType.types.append(np.ndarray)
 
 # class KeyPolicy:
 #     EXACT = "E"  # require exact key match
@@ -763,6 +865,9 @@ TypeRegistry.add(UnionType)
 TypeRegistry.add(ObjectType)
 TypeRegistry.add(ConstType)
 
+# Common Industry Types
+TypeRegistry.add(NDArrayType)
+
 __all__ = [
     "TypeRegistry",
     "InvalidType",
@@ -779,4 +884,5 @@ __all__ = [
     "ConstType",
     "OptionalType",
     "Type",
+    "NDArrayType",
 ]
