@@ -47,8 +47,8 @@ from sentry_sdk import configure_scope
 from wandb.env import error_reporting_enabled
 
 import wandb
+from wandb.errors import CommError
 from wandb.old.core import wandb_dir
-from wandb.errors.error import CommError
 from wandb import env
 
 logger = logging.getLogger(__name__)
@@ -148,10 +148,7 @@ def vendor_setup():
 
     parent_dir = os.path.abspath(os.path.dirname(__file__))
     vendor_dir = os.path.join(parent_dir, "vendor")
-    vendor_packages = (
-        "gql-0.2.0",
-        "graphql-core-1.1",
-    )
+    vendor_packages = ("gql-0.2.0", "graphql-core-1.1")
     package_dirs = [os.path.join(vendor_dir, p) for p in vendor_packages]
     for p in [vendor_dir] + package_dirs:
         if p not in sys.path:
@@ -273,7 +270,10 @@ VALUE_BYTES_LIMIT = 100000
 
 
 def app_url(api_url):
-    if "://api.wandb." in api_url:
+    if "://api.wandb.test" in api_url:
+        # dev mode
+        return api_url.replace("://api.", "://app.")
+    elif "://api.wandb." in api_url:
         # cloud
         return api_url.replace("://api.", "://")
     elif "://api." in api_url:
@@ -467,6 +467,8 @@ def json_friendly(obj):
             if hasattr(obj, "__qualname__") and hasattr(obj, "__module__")
             else str(obj)
         )
+    elif isinstance(obj, float) and math.isnan(obj):
+        obj = None
     else:
         converted = False
     if getsizeof(obj) > VALUE_BYTES_LIMIT:
@@ -723,6 +725,7 @@ def request_with_retry(func, *args, **kwargs):
         **kwargs: passed through to func
     """
     max_retries = kwargs.pop("max_retries", 30)
+    retry_callback = kwargs.pop("retry_callback", None)
     sleep = 2
     retry_count = 0
     while True:
@@ -742,22 +745,35 @@ def request_with_retry(func, *args, **kwargs):
                 # returns them when there are infrastructure issues. If retrying
                 # some request winds up being problematic, we'll change the
                 # back end to indicate that it shouldn't be retried.
-                if e.response.status_code in {400, 403, 404, 409}:
+                if (
+                    e.response is not None
+                    and e.response.status_code in {400, 403, 404, 409}
+                ) or (
+                    e.response is not None
+                    and e.response.status_code == 500
+                    and e.response.content == b'{"error":"context deadline exceeded"}\n'
+                ):
                     return e
 
             if retry_count == max_retries:
                 return e
             retry_count += 1
             delay = sleep + random.random() * 0.25 * sleep
-            if (
-                isinstance(e, requests.exceptions.HTTPError)
-                and e.response.status_code == 429
+            if isinstance(e, requests.exceptions.HTTPError) and (
+                e.response is not None and e.response.status_code == 429
             ):
-                logger.info("Rate limit exceeded, retrying in %s seconds" % delay)
+                err_str = "Filestream rate limit exceeded, retrying in {} seconds".format(
+                    delay
+                )
+                if retry_callback:
+                    retry_callback(e.response.status_code, err_str)
+                logger.info(err_str)
             else:
+                pass
                 logger.warning(
-                    "requests_with_retry encountered retryable exception: %s. args: %s, kwargs: %s",
+                    "requests_with_retry encountered retryable exception: %s. func: %s, args: %s, kwargs: %s",
                     e,
+                    func,
                     args,
                     kwargs,
                 )
@@ -947,11 +963,11 @@ def image_id_from_k8s():
 
 def async_call(target, timeout=None):
     """Accepts a method and optional timeout.
-       Returns a new method that will call the original with any args, waiting for upto timeout seconds.
-       This new method blocks on the original and returns the result or None
-       if timeout was reached, along with the thread.
-       You can check thread.is_alive() to determine if a timeout was reached.
-       If an exception is thrown in the thread, we reraise it.
+    Returns a new method that will call the original with any args, waiting for upto timeout seconds.
+    This new method blocks on the original and returns the result or None
+    if timeout was reached, along with the thread.
+    You can check thread.is_alive() to determine if a timeout was reached.
+    If an exception is thrown in the thread, we reraise it.
     """
     q = queue.Queue()
 
@@ -1105,7 +1121,7 @@ def parse_sweep_id(parts_dict):
 
     Arguments:
         parts_dict (dict): dict(entity=,project=,name=).  Modifies dict inplace.
-    
+
     Returns:
         None or str if there is an error
     """
@@ -1235,6 +1251,11 @@ def _has_internet():
         return False
 
 
+def rand_alphanumeric(length=8, rand=None):
+    rand = rand or random
+    return "".join(rand.choice("0123456789ABCDEF") for _ in range(length))
+
+
 @contextlib.contextmanager
 def fsync_open(path, mode="w"):
     """
@@ -1253,3 +1274,29 @@ def _is_kaggle():
         os.getenv("KAGGLE_KERNEL_RUN_TYPE") is not None
         or "kaggle_environments" in sys.modules  # noqa: W503
     )
+
+
+def _is_likely_kaggle():
+    # Telemetry to mark first runs from Kagglers.
+    return (
+        _is_kaggle()
+        or os.path.exists(
+            os.path.expanduser(os.path.join("~", ".kaggle", "kaggle.json"))
+        )
+        or "kaggle" in sys.modules
+    )
+
+
+def _is_databricks():
+    # check if we are running inside a databricks notebook by
+    # inspecting sys.modules, searching for dbutils and verifying that
+    # it has the appropriate structure
+
+    if "dbutils" in sys.modules:
+        dbutils = sys.modules["dbutils"]
+        if hasattr(dbutils, "shell"):
+            shell = dbutils.shell
+            if hasattr(shell, "sc"):
+                sc = shell.sc
+                return sc.appName == "Databricks Shell"
+    return False
