@@ -6,6 +6,7 @@
 from collections import namedtuple
 import itertools
 import weakref
+import sys
 from six.moves import reduce
 from distutils.version import LooseVersion
 from operator import mul
@@ -306,6 +307,14 @@ class TorchHistory(object):
 class TorchGraph(wandb.data_types.Graph):
     def __init__(self):
         super(TorchGraph, self).__init__("torch")
+        # When we changed to register_full_backward_hook a regression test running fastai v1
+        # started failing.  To maximize compatability we don't use full backward hooks
+        # when we detect fastai v1 has been imported :(
+        self._should_use_full_hooks = True
+        if "fastai" in sys.modules:
+            fastai = util.get_module("fastai")
+            if fastai.__version__.startswith("1."):
+                self._should_use_full_hooks = False
 
     @classmethod
     def hook_torch(cls, model, criterion=None, graph_idx=0):
@@ -432,7 +441,14 @@ class TorchGraph(wandb.data_types.Graph):
                         self.create_forward_hook(name, modules)
                     )
                 )
-                hooks.append(sub_module.register_backward_hook(backward_hook))
+                # Models with dicts as output must use register_full_backward_hook
+                if self._should_use_full_hooks and hasattr(
+                    sub_module, "register_full_backward_hook"
+                ):
+                    hook = sub_module.register_full_backward_hook(backward_hook)
+                else:
+                    hook = sub_module.register_backward_hook(backward_hook)
+                hooks.append(hook)
 
     @classmethod
     def from_torch_layers(cls, module_graph, variable):
